module read_snapshot_m

  implicit none

contains

  subroutine read_snapshot(s, hshp, nlon, nlat, d_init, k, corner, step, copy, &
       ishape_last)

    ! Note: read_field_indices must have been called before read_snapshot.

    ! Libraries:
    use contour_531, only: convert_to_ind 
    use nr_util, only: assert

    use derived_types, only: snapshot, eddy, shpc
    use read_eddy_m, only: read_eddy

    type(snapshot), intent(out):: s ! completely defined
    TYPE(shpc), intent(in):: hshp

    integer, intent(in):: nlon, nlat
    ! size of ssh array in input NetCDF, assuming no repeated point if
    ! the grid is global

    integer, intent(in):: d_init
    integer, intent(in):: k ! date index

    real, intent(in):: corner(:) ! (2) longitude and latitude of the
    ! corner of the whole grid in input NetCDF, in rad

    real, intent(in):: step(:) ! (2) longitude and latitude steps, in rad

    integer, intent(in):: copy, ishape_last(d_init:)

    ! Local:
    integer ishape, ishape_first
    integer date_read
    integer eddy_i ! eddy index
    type(eddy) e

    !---------------------------------------------------------------------

    if (k == d_init) then
       ishape_first = 0
    else
       ! {k > d_init}
       ishape_first = ishape_last(k - 1) + 1
    end if

    s%number_extr = ishape_last(k) - ishape_first + 1
    allocate(s%list(s%number_extr))

    do ishape = ishape_first, ishape_last(k)
       call read_eddy(e, date_read, eddy_i, hshp, ishape)

       ! Check that all the eddies have the same date index:
       if (date_read /= k) then
          print *, "read_snapshot: bad date index"
          print *, "date_read = ", date_read
          print *, "k = ", k
          print *, "ishape = ", ishape
          print *, "eddy_i = ", eddy_i
          stop 1
       end if

       s%list(eddy_i) = e
    end do

    allocate(s%ind_extr(2, s%number_extr))
    forall (eddy_i = 1:s%number_extr) s%ind_extr(:, eddy_i) &
         = nint(convert_to_ind(s%list(eddy_i)%coord_extr, corner, step))

    ! Define s%extr_map from s%ind_extr:

    allocate(s%extr_map(1 - copy:nlon + copy, nlat))
    s%extr_map(1:nlon, :) = 0

    ! Since we are not sure that there is no duplicate tuple in
    ! s%ind_extr, we must use a do construct and not a forall:
    do eddy_i = 1, s%number_extr
       s%extr_map(s%ind_extr(1, eddy_i), s%ind_extr(2, eddy_i)) = eddy_i
    end do

    call assert(count(s%extr_map(1:nlon, :) /= 0) == s%number_extr, &
         "read_snapshot: one eddy per grid point")

    ! Extend in longitude:
    s%extr_map(1 - copy:0, :) = s%extr_map(nlon + 1 - copy:nlon, :)
    s%extr_map(nlon + 1:nlon + copy, :) = s%extr_map(1:copy, :)

  end subroutine read_snapshot
  
end module read_snapshot_m
