program test_set_all_outerm

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_get_var, find_coord, &
       nf95_inquire_dimension
  use nr_util, only: pi, assert, deg_to_rad, twopi

  use config_m, only: config, max_radius_deg, min_radius
  use derived_types, only: snapshot, shpc, null_ssh_contour, missing_speed
  use get_var_m, only: get_var
  use set_all_outerm_m, only: set_all_outerm
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use write_eddy_m, only: write_eddy

  implicit none

  type(snapshot) s
  character(len = :), allocatable:: filename
  integer nlon, nlat, copy
  real lon_min, lon_max, lat_min, lat_max ! longitudes and latitudes, in degrees

  integer max_radius(2)
  ! maximum radius of an eddy in longitude and latitude, in number of
  ! grid points

  integer ncid, varid, dimid, i, n_cyclo, n_anti
  real, allocatable:: ssh(:, :)
  ! (1 - max_radius(1):nlon + max_radius(1), nlat) if the grid is periodic
  ! in longitude, else (nlon, nlat). Sea-surface height, in m.

  real step_deg(2), step(2) ! longitude and latitude steps, in degrees and rad
  logical periodic ! grid is periodic in longitude
  TYPE(shpc) hshp_cyclo, hshp_anti

  !--------------------------------------------------------------

  call get_command_arg_dyn(1, filename, &
       "Required argument: NetCDF file containing adt")
  call config(exist = .false.)

  ! Read ssh:

  call nf95_open(filename, nf90_nowrite, ncid)

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "longitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlon)
  call nf95_get_var(ncid, varid, lon_min)
  call nf95_get_var(ncid, varid, lon_max, start = [nlon])

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "latitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlat)
  call nf95_get_var(ncid, varid, lat_min)
  call nf95_get_var(ncid, varid, lat_max, start = [nlat])

  call assert(lon_max > lon_min .and. lat_max > lat_min, &
       "test_set_all_outerm coordinate order")
  step_deg = [(lon_max - lon_min) / (nlon - 1), &
       (lat_max - lat_min) / (nlat - 1)]
  step = step_deg * deg_to_rad
  max_radius = nint(max_radius_deg / step_deg)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step(1) and nlon:
  periodic = nint(twopi / step(1)) == nlon
  print *, "periodic = ", periodic
  call assert(2 * max_radius(1) * step(1) < pi, &
       "test_set_all_outerm max_radius")

  copy = merge(max_radius(1), 0, periodic)
  allocate(ssh(1 - copy:nlon + copy, nlat))
  call get_var(periodic, max_radius(1), ssh, ncid, nlon, name = "adt", &
       new_fill_value = huge(0.))

  call nf95_close(ncid)

  call set_all_outerm(s, max_radius, step, periodic, ssh, &
       corner = [lon_min, lat_min] * deg_to_rad, &
       min_area = pi * (min_radius * 1e3)**2)

  do i = 1, s%number_extr
     s%list(i)%speed_cont = null_ssh_contour()
     s%list(i)%max_speed = missing_speed
     s%list(i)%radius4 = 0
  end do

  call shpc_create(hshp_cyclo, shpc_dir = "SHPC_cyclo", cyclone = .true.)
  call shpc_create(hshp_anti, shpc_dir = "SHPC_anti", cyclone = .false.)
  n_cyclo = 0
  n_anti = 0

  do i = 1, s%number_extr
     if (s%list(i)%cyclone) then
        n_cyclo = n_cyclo + 1
        call write_eddy(s%list(i), hshp_cyclo, date = 1, i = n_cyclo)
     else
        n_anti = n_anti + 1
        call write_eddy(s%list(i), hshp_anti, date = 1, i = n_anti)
     end if
  end do

  print *, "s%number_extr = ", s%number_extr

  CALL shpc_close(hshp_cyclo)
  CALL shpc_close(hshp_anti)
  print *, 'Created shapefile collections in SHPC_cyclo and SHPC_anti.'
  print *, "Average number of points per outermost contour: ", &
       sum(s%list%out_cont%n_points) / real(s%number_extr)

end program test_set_all_outerm
