program test_set_max_speed

  use, intrinsic:: ISO_FORTRAN_ENV
  use, intrinsic:: ieee_arithmetic, only: ieee_value, IEEE_QUIET_NAN


  ! Libraries:
  use jumble, only: new_unit, count_lines, get_command_arg_dyn
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       find_coord, nf95_inquire_dimension
  use nr_util, only: deg_to_rad

  use derived_types, only: eddy, shpc
  use get_var_m, only: get_var
  use read_eddy_m, only: read_eddy
  use set_max_speed_m, only: set_max_speed
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use shpc_open_m, only: shpc_open
  use write_eddy_m, only: write_eddy

  implicit none

  integer ncid, varid, dimid
  integer nlon, nlat, unit, n, l, d, i
  real, allocatable:: ssh(:, :) ! (nlon, nlat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  type(eddy) e
  real corner(2), step(2) ! in rad
  TYPE(shpc) hshp
  character(len = :), allocatable:: shpc_dir

  real, allocatable:: outside_points(:, :) ! (2, :) longitude and
  ! latitude, in rad, of all the extrema except the target extremum

  !----------------------------------------------------------------

  call get_command_arg_dyn(1, shpc_dir, "Required argument: SHPC-directory")
  print *, "Reading from h.nc..."
  call nf95_open("h.nc", nf90_nowrite, ncid)

  ! Read corner:

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "longitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlon)
  call nf95_get_var(ncid, varid, corner(1))

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "latitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlat)
  call nf95_get_var(ncid, varid, corner(2))

  corner = corner * deg_to_rad

  allocate(ssh(nlon, nlat))
  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_get_var(ncid, varid, ssh)

  call nf95_close(ncid)

  print *, "Reading from uv.nc..."
  call nf95_open("uv.nc", nf90_nowrite, ncid)
  allocate(u(nlon, nlat), v(nlon, nlat))
  call get_var(periodic = .false., max_rad_lon = 0, values = u, ncid = ncid, &
       nlon = nlon, name = "ugos", &
       new_fill_value = ieee_value(0., IEEE_QUIET_NAN))
  call get_var(periodic = .false., max_rad_lon = 0, values = v, ncid = ncid, &
       nlon = nlon, name = "vgos", &
       new_fill_value = ieee_value(0., IEEE_QUIET_NAN))
  ! (We will need quiet NaNs rather the original fill values for u and
  ! v when we compute the max-speed contours and when we search the
  ! ssh of max-speed contours.)
  call nf95_close(ncid)

  print *, "Reading from shapefiles..."
  call shpc_open(hshp, shpc_dir, rank = 0, pszaccess = "rb")
  call read_eddy(e, d, i, hshp, ishape = 0)
  CALL shpc_close(hshp)

  print *, "Reading from outside_points.csv..."
  call new_unit(unit)
  open(unit, file = "outside_points.csv", status = "old", action = "read", &
       position = "rewind")
  call count_lines(unit, n)
  n = n - 2 ! 2 title lines
  allocate(outside_points(2, n))
  rewind(unit)
  read(unit, fmt = *)
  read(unit, fmt = *)

  do l = 1, n
     read(unit, fmt = *) outside_points(:, l)
  end do
  
  outside_points = outside_points * deg_to_rad
  close(unit)
  step = 0.25  * deg_to_rad
  call set_max_speed(e, nint((e%coord_extr - corner) / step) + 1, &
       outside_points, ssh, u, v, corner, step)
  call shpc_create(hshp, shpc_dir = "SHPC", cyclone = e%cyclone)
  call write_eddy(e, hshp, d, i)
  CALL shpc_close(hshp)
  print *, 'Created "SHPC".'

end program test_set_max_speed
