program test_overlap

  use, intrinsic:: ISO_FORTRAN_ENV

  ! Libraries:
  use jumble, only: get_command_arg_dyn, read_column, new_unit, ediff1d
  use nr_util, only: deg_to_rad, assert
  use shapelib_03, only: dbf_read_attribute_03

  use derived_types, only: snapshot, shpc
  use overlap_m, only: overlap
  use read_snapshot_m, only: read_snapshot
  use shpc_close_m, only: shpc_close
  use shpc_open_m, only: shpc_open
  use unit_edge_m, only: open_edge_file, unit_edge

  implicit none

  character(len = :), allocatable:: shpc_dir
  integer d_init
  integer:: k_test_1 = 0, k_test_2 = 1
  integer unit, i, copy, n_dates
  integer, allocatable:: ishape_last(:)
  type(snapshot), allocatable:: flow(:) ! (max_delta + 1)
  TYPE(shpc) hshp

  real:: corner_deg(2) = [huge(0.), huge(0.)], corner(2)
  ! longitude and latitude of the corner of the whole grid, in degrees
  ! and in rad

  real:: step_deg(2) = [huge(0.), huge(0.)], step(2)
  ! longitude and latitude steps, in degrees and in rad

  logical periodic ! grid is periodic in longitude
  integer:: nlon = - 1, nlat = - 1, max_delta = 1
  integer e_overestim ! over-estimation of the number of eddies at each date

  integer:: dist_lim = 12
  ! We look for an overlapping eddy at dist_lim (in grid points) of
  ! the first extremum.

  namelist /grid_nml/ corner_deg, step_deg, nlon, nlat
  namelist /main_nml/ dist_lim, max_delta, k_test_1, k_test_2

  !-------------------------------------------------------------------------

  call get_command_arg_dyn(1, shpc_dir, "Required argument: SHPC-directory")
  call new_unit(unit)
  open(unit, file = shpc_dir // "/grid_nml.txt", status = "old", &
       action = "read", position = "rewind")
  read(unit, nml = grid_nml)
  close(unit)
  write(unit = error_unit, nml = main_nml)
  write(unit = error_unit, fmt = *) "Enter namelist main_nml."
  read(unit = *, nml = main_nml)
  write(unit = *, nml = main_nml)

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" may be deduced from the values of step_deg(1) and nlon:
  periodic = nint(360. / step_deg(1)) == nlon
  print *, "periodic = ", periodic
  if (periodic) call assert(2 * dist_lim * step_deg(1) < 180., &
       "test_overlap dist_lim")
  copy = merge(dist_lim, 0, periodic)

  corner = corner_deg * deg_to_rad
  step = step_deg * deg_to_rad
  allocate(flow(max_delta + 1))
  call shpc_open(hshp, trim(shpc_dir), rank = 0, pszaccess = "rb")
  call dbf_read_attribute_03(d_init, hshp%extremum, hshp%extr_date, ishape = 0)
  call read_column(ishape_last, file = trim(shpc_dir) // "/ishape_last.txt", &
       my_lbound = d_init)
  n_dates = size(ishape_last)
  call assert(d_init <= [k_test_1, k_test_2] .and. [k_test_1, k_test_2] &
       < d_init + n_dates, "test_overlap k_test_1, k_test_2")
  e_overestim = maxval([ishape_last(d_init) + 1, ediff1d(ishape_last)])
  open(unit, file = "node_id_param.json", status = "replace", &
       action = "write")
  write(unit, fmt = *) '{"e_overestim": ', e_overestim, '}'
  close(unit)
  call read_snapshot(flow(1), hshp, nlon, nlat, d_init, k_test_1, corner, &
       step, copy, ishape_last)
  call read_snapshot(flow(max_delta + 1), hshp, nlon, nlat, d_init, k_test_2, &
       corner, step, copy, ishape_last)
  CALL shpc_close(hshp)
  print *, "Enter flow(1)%list%delta_out (array with ", &
       flow(1)%number_extr, " values):"
  read *, flow(1)%list%delta_out
  print *, "Enter flow(max_delta + 1)%list%delta_in (array with ", &
       flow(max_delta + 1)%number_extr, " values)):"
  read *, flow(max_delta + 1)%list%delta_in
  call open_edge_file(rank = 0)
  call overlap(flow, nlon, nlat, periodic, dist_lim, e_overestim, &
       k = k_test_2, delta = max_delta, j = max_delta + 1)
  close(unit_edge)
  print *, 'Created file "edgelist.csv".'

  print *, k_test_1, ":"
  print *, "Valid isolated eddies:"
  do i = 1, flow(1)%number_extr
     if (flow(1)%list(i)%valid .and. flow(1)%list(i)%delta_out &
          == huge(0)) write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do
  print *

  print *
  print *, k_test_2, ":"
  print *, "Valid isolated eddies:"

  do i = 1, flow(max_delta + 1)%number_extr
     if (flow(max_delta + 1)%list(i)%valid &
          .and. flow(max_delta + 1)%list(i)%delta_in == huge(0)) &
          write(unit = *, fmt = "(i0, 1x)", advance = "no") i
  end do

  print *

end program test_overlap
